@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.529-3-gfbc5ef2e
+package JSON::Schema::Modern; # git description: v0.530-10-g3555f8b5
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema data validation structure specification
 
-our $VERSION = '0.530';
+our $VERSION = '0.531';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -27,7 +27,7 @@ use File::ShareDir 'dist_dir';
 use Module::Runtime qw(use_module require_module);
 use MooX::TypeTiny 0.002002;
 use MooX::HandlesVia;
-use Types::Standard 1.010002 qw(Bool Int Str HasMethods Enum InstanceOf HashRef Dict CodeRef Optional slurpy ArrayRef Undef ClassName Tuple);
+use Types::Standard 1.010002 qw(Bool Int Str HasMethods Enum InstanceOf HashRef Dict CodeRef Optional slurpy ArrayRef Undef ClassName Tuple Map);
 use Feature::Compat::Try;
 use JSON::Schema::Modern::Error;
 use JSON::Schema::Modern::Result;
@@ -100,7 +100,7 @@ has scalarref_booleans => (
 
 has _format_validations => (
   is => 'bare',
-  isa => Dict[
+  isa => my $format_type = Dict[
     (map +($_ => Optional[CodeRef]), qw(date-time date time duration email idn-email hostname idn-hostname ipv4 ipv6 uri uri-reference iri iri-reference uuid uri-template json-pointer relative-json-pointer regex)),
     slurpy HashRef[Dict[type => Enum[qw(null object array boolean string number integer)], sub => CodeRef]],
   ],
@@ -108,11 +108,14 @@ has _format_validations => (
   handles_via => 'Hash',
   handles => {
     _get_format_validation => 'get',
+    add_format_validation => 'set',
   },
   lazy => 1,
   default => sub { {} },
 );
 
+before add_format_validation => sub ($self, $name, $sub) { $format_type->({ $name => $sub }) };
+
 around BUILDARGS => sub ($orig, $class, @args) {
   my $args = $class->$orig(@args);
   croak 'output_format: strict_basic can only be used with specification_version: draft2019-09'
@@ -311,7 +314,7 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
         schema => $document->schema,
         document => $document,
         document_path => '',
-        (map +($_ => $base_resource->{$_}), qw(canonical_uri specification_version vocabularies configs)),
+        $base_resource->%{qw(canonical_uri specification_version vocabularies configs)},
       };
     }
 
@@ -767,6 +770,7 @@ sub _get_or_load_resource ($self, $uri) {
 # - the path relative to the document root for this schema
 # - the specification version that applies to this schema
 # - the vocabularies to use when considering schema keywords
+# - the config overrides to set when considering schema keywords
 # creates a Document and adds it to the resource index, if not already present.
 sub _fetch_from_uri ($self, $uri) {
   $uri = Mojo::URL->new($uri) if not is_ref($uri);
@@ -819,13 +823,15 @@ has _json_decoder => (
   default => sub { JSON::MaybeXS->new(allow_nonref => 1, canonical => 1, utf8 => 1) },
 );
 
+# since media types are case-insensitive, all type names must be foldcased on insertion.
 has _media_type => (
   is => 'bare',
-  isa => HashRef[CodeRef],
+  isa => my $media_type_type = Map[Str->where(q{$_ eq CORE::fc($_)}), CodeRef],
   handles_via => 'Hash',
   handles => {
     get_media_type => 'get',
     add_media_type => 'set',
+    _media_types => 'keys',
   },
   lazy => 1,
   default => sub ($self) {
@@ -834,11 +840,24 @@ has _media_type => (
       'application/json' => sub ($content_ref) {
         \ JSON::MaybeXS->new(allow_nonref => 1, utf8 => 0)->decode($content_ref->$*);
       },
-      'text/plain' => sub ($content_ref) { $content_ref }
+      map +($_ => sub ($content_ref) { $content_ref }),
+        qw(text/plain application/octet-stream),
     };
   },
 );
 
+# get_media_type('TExT/bloop') will match an entry for 'text/*' or '*/*'
+# TODO: support queries for application/schema+json to match entry of application/json
+around get_media_type => sub ($orig, $self, $type) {
+  my $mt = $self->$orig(fc $type);
+  return $mt if $mt;
+
+  return $self->$orig((first { m{([^/]+)/\*$} && fc($type) =~ m{^\Q$1\E/[^/]+$} } $self->_media_types)
+    // '*/*');
+};
+
+before add_media_type => sub ($self, $type, $sub) { $media_type_type->({ $type => $sub }) };
+
 has _encoding => (
   is => 'bare',
   isa => HashRef[CodeRef],
@@ -869,18 +888,11 @@ sub FREEZE ($self, $serializer) {
 }
 
 sub THAW ($class, $serializer, $data) {
-  # sadly I can't do %bare_attrs = delete ...->%{...} until 5.28
-  my %bare_attrs;
-  @bare_attrs{qw(_resource_index _vocabulary_classes _metaschema_vocabulary_classes)}
-    = delete $data->@{qw(_resource_index _vocabulary_classes _metaschema_vocabulary_classes)};
-
-  my $self = $class->new($data);
-  $self->_add_resources_unsafe($bare_attrs{_resource_index}->%*);
-  $self->_set_vocabulary_class($bare_attrs{_vocabulary_classes}->%*);
-  $self->_set_metaschema_vocabulary_classes($bare_attrs{_metaschema_vocabulary_classes}->%*);
+  my $self = bless($data, $class);
 
   # load all vocabulary classes
   require_module($_) foreach uniq map $_->{vocabularies}->@*, $self->_canonical_resources;
+
   return $self;
 }
 
@@ -900,7 +912,7 @@ JSON::Schema::Modern - Validate data against a schema
 
 =head1 VERSION
 
-version 0.530
+version 0.531
 
 =head1 SYNOPSIS
 
@@ -976,10 +988,12 @@ to false.
 
 =head2 format_validations
 
+=for stopwords subref
+
 An optional hashref that allows overriding the validation method for formats, or adding new ones.
 Overrides to existing formats (see L</Format Validation>)
 must be specified in the form of C<< { $format_name => $format_sub } >>, where
-the format sub is a coderef that takes one argument and returns a boolean result. New formats must
+the format sub is a subref that takes one argument and returns a boolean result. New formats must
 be specified in the form of C<< { $format_name => { type => $type, sub => $format_sub } } >>,
 where the type indicates which of the core JSON Schema types (null, object, array, boolean, string,
 number, or integer) the instance value must be for the format validation to be considered.
@@ -1152,6 +1166,16 @@ Returns C<undef> if the resource could not be found;
 if there were errors in the document, will die with these errors;
 otherwise returns the L<JSON::Schema::Modern::Document> that contains the added schema.
 
+=head2 add_format_validation
+
+=for comment we are the nine Eleven Deniers
+
+  $js->add_format_validation(no_nines => { type => 'number', sub => sub ($value) { $value =~ m/^[0-8]$$/ });
+
+Adds support for a custom format. The data type that this format applies to must be supplied; all
+values of any other type will automatically be deemed to be valid, and will not be passed to the
+subref.
+
 =head2 add_vocabulary
 
   $js->add_vocabulary('My::Custom::Vocabulary::Class');
@@ -1170,8 +1194,6 @@ L<keywords|JSON::Schema::Modern::Vocabulary/keywords> methods.
     return ...;  # data representing the deserialized text for Content-Type: application/furble
   });
 
-=for stopwords subref
-
 Takes a media-type name and a subref which takes a single scalar reference, which is expected to be
 a reference to a string, which might contain wide characters (i.e. not octets), especially when used
 in conjunction with L</get_encoding> below. Must return B<a reference to a value of any type> (which is
@@ -1193,7 +1215,7 @@ C<text/plain>
 
 =head2 get_media_type
 
-Fetches a decoder sub for the indicated media type.
+Fetches a decoder sub for the indicated media type. Lookups are performed B<without case sensitivity>.
 
 =for stopwords thusly
 
