@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.530';
+our $VERSION = '0.531';
 
 use 5.020;
 use Moo;
@@ -114,10 +114,11 @@ around _add_resources => sub {
   my $orig = shift;
   my $self = shift;
 
-  $resource_type->($_[1]) if @_;  # check type of hash value against Dict
-
   foreach my $pair (pairs @_) {
     my ($key, $value) = @$pair;
+
+    $resource_type->($value); # check type of hash value against Dict
+
     if (my $existing = $self->_get_resource($key)) {
       croak 'uri "'.$key.'" conflicts with an existing schema resource'
         if $existing->{path} ne $value->{path}
@@ -208,7 +209,7 @@ JSON::Schema::Modern::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.530
+version 0.531
 
 =head1 SYNOPSIS
 
